@@ -1,14 +1,13 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2011-2017 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011-2021 -- leonerd@leonerd.org.uk
 
-package Tangence::Meta::Method 0.26;
+use v5.26;
+use Object::Pad 0.51;
 
-use v5.14;
-use warnings;
-
-use Scalar::Util qw( weaken );
+package Tangence::Meta::Method 0.27;
+class Tangence::Meta::Method :strict(params);
 
 =head1 NAME
 
@@ -55,14 +54,15 @@ reference
 
 =cut
 
-sub new
+has $class :param :weak :reader;
+has $name  :param       :reader;
+has @arguments;
+has $ret   :param       :reader;
+
+ADJUSTPARAMS ( $params )
 {
-   my $class = shift;
-   my %args = @_;
-   $args{arguments} ||= [];
-   my $self = bless \%args, $class;
-   weaken $self->{class};
-   return $self;
+   exists $params->{arguments} and
+      @arguments = @{ delete $params->{arguments} };
 }
 
 =head1 ACCESSORS
@@ -77,12 +77,6 @@ Returns the class the method is a member of
 
 =cut
 
-sub class
-{
-   my $self = shift;
-   return $self->{class};
-}
-
 =head2 name
 
    $name = $method->name
@@ -91,12 +85,6 @@ Returns the name of the class
 
 =cut
 
-sub name
-{
-   my $self = shift;
-   return $self->{name};
-}
-
 =head2 arguments
 
    @arguments = $method->arguments
@@ -105,11 +93,7 @@ Return the arguments in a list of L<Tangence::Meta::Argument> references.
 
 =cut
 
-sub arguments
-{
-   my $self = shift;
-   return @{ $self->{arguments} };
-}
+method arguments { @arguments }
 
 =head2 argtype
 
@@ -119,10 +103,9 @@ Return the argument types in a list of L<Tangence::Meta::Type> references.
 
 =cut
 
-sub argtypes
+method argtypes
 {
-   my $self = shift;
-   return map { $_->type } $self->arguments;
+   return map { $_->type } @arguments;
 }
 
 =head2 ret
@@ -134,12 +117,6 @@ the method does not return a value.
 
 =cut
 
-sub ret
-{
-   my $self = shift;
-   return $self->{ret};
-}
-
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
