@@ -1,15 +1,21 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2010-2020 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2010-2021 -- leonerd@leonerd.org.uk
 
-package Tangence::ObjectProxy 0.26;
+use v5.26;
+use Object::Pad 0.41;
 
-use v5.14;
-use warnings;
+package Tangence::ObjectProxy 0.27;
+class Tangence::ObjectProxy;
 
 use Carp;
 
+use Syntax::Keyword::Match 0.06;
+
+use Future::AsyncAwait;
+use Future::Exception;
+
 use Tangence::Constants;
 
 use Tangence::Types;
@@ -35,53 +41,36 @@ object.
 
 =cut
 
-sub new
+has $_client :param :weak :reader;
+has $_id     :param       :reader;
+has $_class  :param       :reader;
+
+has $_destroyed;
+
+has %_subscriptions;
+has %_props;
+
+method destroy
 {
-   my $class = shift;
-   my %args = @_;
+   $_destroyed = 1;
 
-   my $self = bless {
-      client => $args{client},
-      id     => $args{id},
-
-      class => $args{class},
-
-      on_error => $args{on_error},
-   }, $class;
-
-   # An ObjectProxy is useless after its connection disappears
-   weaken( $self->{client} );
-
-   return $self;
-}
-
-sub destroy
-{
-   my $self = shift;
-
-   $self->{destroyed} = 1;
-
-   foreach my $cb ( @{ $self->{subscriptions}->{destroy} } ) {
+   foreach my $cb ( @{ $_subscriptions{destroy} } ) {
       $cb->();
    }
-
-   undef %$self;
-   $self->{destroyed} = 1;
 }
 
 =head1 METHODS
 
-The following methods documented with a trailing call to C<< ->get >> return
-L<Future> instances.
+The following methods documented in an C<await> expression return L<Future>
+instances.
 
 =cut
 
 use overload '""' => \&STRING;
 
-sub STRING
+method STRING
 {
-   my $self = shift;
-   return "Tangence::ObjectProxy[id=$self->{id}]";
+   return "Tangence::ObjectProxy[id=$_id]";
 }
 
 =head2 id
@@ -92,11 +81,7 @@ Returns the object ID for the C<Tangence> object being proxied for.
 
 =cut
 
-sub id
-{
-   my $self = shift;
-   return $self->{id};
-}
+# generated accessor
 
 =head2 classname
 
@@ -106,10 +91,9 @@ Returns the name of the class of the C<Tangence> object being proxied for.
 
 =cut
 
-sub classname
+method classname
 {
-   my $self = shift;
-   return $self->{class}->name;
+   return $_class->name;
 }
 
 =head2 class
@@ -121,11 +105,7 @@ object.
 
 =cut
 
-sub class
-{
-   my $self = shift;
-   return $self->{class};
-}
+# generated accessor
 
 =head2 can_method
 
@@ -136,10 +116,9 @@ C<undef> if no such method exists.
 
 =cut
 
-sub can_method
+method can_method
 {
-   my $self = shift;
-   return $self->class->method( @_ );
+   return $_class->method( @_ );
 }
 
 =head2 can_event
@@ -151,10 +130,9 @@ C<undef> if no such event exists.
 
 =cut
 
-sub can_event
+method can_event
 {
-   my $self = shift;
-   return $self->class->event( @_ );
+   return $_class->event( @_ );
 }
 
 =head2 can_property
@@ -166,30 +144,25 @@ property, or C<undef> if no such property exists.
 
 =cut
 
-sub can_property
+method can_property
 {
-   my $self = shift;
-   return $self->class->property( @_ );
+   return $_class->property( @_ );
 }
 
 # Don't want to call it "isa"
-sub proxy_isa
+method proxy_isa
 {
-   my $self = shift;
    if( @_ ) {
       my ( $class ) = @_;
-      return !! grep { $_->name eq $class } $self->{class}, $self->{class}->superclasses;
+      return !! grep { $_->name eq $class } $_class, $_class->superclasses;
    }
    else {
-      return $self->{class}, $self->{class}->superclasses
+      return $_class, $_class->superclasses
    }
 }
 
-sub grab
+method grab ( $smashdata )
 {
-   my $self = shift;
-   my ( $smashdata ) = @_;
-
    foreach my $property ( keys %{ $smashdata } ) {
       my $value = $smashdata->{$property};
       my $dim = $self->can_property( $property )->dimension;
@@ -199,25 +172,22 @@ sub grab
          $value = { map { $_->id => $_ } @$value };
       }
 
-      my $prop = $self->{props}->{$property} ||= {};
+      my $prop = $_props{$property} ||= {};
       $prop->{cache} = $value;
    }
 }
 
 =head2 call_method
 
-   $result = $proxy->call_method( $mname, @args )->get
+   $result = await $proxy->call_method( $mname, @args )
 
 Calls the given method on the server object, passing in the given arguments.
 Returns a L<Future> that will yield the method's result.
 
 =cut
 
-sub call_method
+async method call_method ( $method, @args )
 {
-   my $self = shift;
-   my ( $method, @args ) = @_;
-
    # Detect void-context legacy uses
    defined wantarray or
       croak "->call_method in void context no longer useful - it now returns a Future";
@@ -225,36 +195,30 @@ sub call_method
    my $mdef = $self->can_method( $method )
       or croak "Class ".$self->classname." does not have a method $method";
 
-   my $client = $self->{client};
-
-   my $request = Tangence::Message->new( $client, MSG_CALL )
+   my $request = Tangence::Message->new( $_client, MSG_CALL )
          ->pack_int( $self->id )
          ->pack_str( $method );
 
    my @argtypes = $mdef->argtypes;
    $argtypes[$_]->pack_value( $request, $args[$_] ) for 0..$#argtypes;
 
-   $client->request(
-      request => $request,
-   )->then( sub {
-      my ( $message ) = @_;
+   my $message = await $_client->request( request => $request );
 
-      my $code = $message->code;
+   my $code = $message->code;
 
-      if( $code == MSG_RESULT ) {
-         my $result = $mdef->ret ? $mdef->ret->unpack_value( $message )
-                                 : undef;
-         Future->done( $result );
-      }
-      else {
-         Future->fail( "Unexpected response code $code", tangence => );
-      }
-   });
+   if( $code == MSG_RESULT ) {
+      my $result = $mdef->ret ? $mdef->ret->unpack_value( $message )
+                              : undef;
+      return $result;
+   }
+   else {
+      Future::Exception->throw( "Unexpected response code $code", tangence => );
+   }
 }
 
 =head2 subscribe_event
 
-   $proxy->subscribe_event( $event, %callbacks )->get
+   await $proxy->subscribe_event( $event, %callbacks )
 
 Subscribes to the given event on the server object, installing a callback
 function which will be invoked whenever the event is fired.
@@ -276,11 +240,8 @@ of the C<on_fire> event handler.
 
 =cut
 
-sub subscribe_event
+async method subscribe_event ( $event, %args )
 {
-   my $self = shift;
-   my ( $event, %args ) = @_;
-
    # Detect void-context legacy uses
    defined wantarray or
       croak "->subscribe_event in void context no longer useful - it now returns a Future";
@@ -291,46 +252,40 @@ sub subscribe_event
    $self->can_event( $event )
       or croak "Class ".$self->classname." does not have an event $event";
 
-   if( my $cbs = $self->{subscriptions}->{$event} ) {
+   if( my $cbs = $_subscriptions{$event} ) {
       push @$cbs, $callback;
-      return Future->done;
+      return;
    }
 
    my @cbs = ( $callback );
-   $self->{subscriptions}->{$event} = \@cbs;
+   $_subscriptions{$event} = \@cbs;
 
-   return Future->done if $event eq "destroy"; # This is automatically handled
+   return if $event eq "destroy"; # This is automatically handled
 
-   my $client = $self->{client};
-
-   $client->request(
-      request => Tangence::Message->new( $client, MSG_SUBSCRIBE )
+   my $message = await $_client->request(
+      request => Tangence::Message->new( $_client, MSG_SUBSCRIBE )
          ->pack_int( $self->id )
-         ->pack_str( $event ),
-   )->then( sub {
-      my ( $message ) = @_;
-      my $code = $message->code;
+         ->pack_str( $event )
+   );
 
-      if( $code == MSG_SUBSCRIBED ) {
-         Future->done;
-      }
-      else {
-         Future->fail( "Unexpected response code $code", tangence => );
-      }
-   });
+   my $code = $message->code;
+
+   if( $code == MSG_SUBSCRIBED ) {
+      return;
+   }
+   else {
+      Future::Exception->throw( "Unexpected response code $code", tangence => );
+   }
 }
 
-sub handle_request_EVENT
+method handle_request_EVENT ( $message )
 {
-   my $self = shift;
-   my ( $message ) = @_;
-
    my $event = $message->unpack_str();
    my $edef = $self->can_event( $event ) or return;
 
    my @args = map { $_->unpack_value( $message ) } $edef->argtypes;
 
-   if( my $cbs = $self->{subscriptions}->{$event} ) {
+   if( my $cbs = $_subscriptions{$event} ) {
       foreach my $cb ( @$cbs ) { $cb->( @args ) }
    }
 }
@@ -344,19 +299,15 @@ previously installed using C<subscribe_event>.
 
 =cut
 
-sub unsubscribe_event
+method unsubscribe_event ( $event )
 {
-   my $self = shift;
-   my ( $event ) = @_;
-
    $self->can_event( $event )
       or croak "Class ".$self->classname." does not have an event $event";
 
    return if $event eq "destroy"; # This is automatically handled
 
-   my $client = $self->{client};
-   $client->request(
-      request => Tangence::Message->new( $client, MSG_UNSUBSCRIBE )
+   $_client->request(
+      request => Tangence::Message->new( $_client, MSG_UNSUBSCRIBE )
          ->pack_int( $self->id )
          ->pack_str( $event ),
 
@@ -366,17 +317,14 @@ sub unsubscribe_event
 
 =head2 get_property
 
-   $value = $proxy->get_property( $prop )->get
+   await $value = $proxy->get_property( $prop )
 
 Requests the current value of the property from the server object.
 
 =cut
 
-sub get_property
+async method get_property ( $property )
 {
-   my $self = shift;
-   my ( $property ) = @_;
-
    # Detect void-context legacy uses
    defined wantarray or
       croak "->get_property in void context no longer useful - it now returns a Future";
@@ -384,39 +332,33 @@ sub get_property
    my $pdef = $self->can_property( $property )
       or croak "Class ".$self->classname." does not have a property $property";
 
-   my $client = $self->{client};
-   $client->request(
-      request => Tangence::Message->new( $client, MSG_GETPROP )
+   my $message = await $_client->request(
+      request => Tangence::Message->new( $_client, MSG_GETPROP )
          ->pack_int( $self->id )
          ->pack_str( $property ),
-   )->then( sub {
-      my ( $message ) = @_;
-      my $code = $message->code;
+   );
 
-      if( $code == MSG_RESULT ) {
-         my $value = $pdef->overall_type->unpack_value( $message );
-         Future->done( $value );
-      }
-      else {
-         Future->fail( "Unexpected response code $code", tangence => );
-      }
-   });
+   my $code = $message->code;
+
+   if( $code == MSG_RESULT ) {
+      return $pdef->overall_type->unpack_value( $message );
+   }
+   else {
+      Future::Exception->throw( "Unexpected response code $code", tangence => );
+   }
 }
 
 =head2 get_property_element
 
-   $value = $proxy->get_property_element( $property, $index_or_key )->get
+   await $value = $proxy->get_property_element( $property, $index_or_key )
 
 Requests the current value of an element of the property from the server
 object.
 
 =cut
 
-sub get_property_element
+async method get_property_element ( $property, $index_or_key )
 {
-   my $self = shift;
-   my ( $property, $index_or_key ) = @_;
-
    # Detect void-context legacy uses
    defined wantarray or
       croak "->get_property_element in void context no longer useful - it now returns a Future";
@@ -424,36 +366,34 @@ sub get_property_element
    my $pdef = $self->can_property( $property )
       or croak "Class ".$self->classname." does not have a property $property";
 
-   my $client = $self->{client};
-
-   my $request = Tangence::Message->new( $client, MSG_GETPROPELEM )
+   my $request = Tangence::Message->new( $_client, MSG_GETPROPELEM )
       ->pack_int( $self->id )
       ->pack_str( $property );
 
-   if( $pdef->dimension == DIM_HASH ) {
-      $request->pack_str( $index_or_key );
+   match( $pdef->dimension : == ) {
+      case( DIM_HASH ) {
+         $request->pack_str( $index_or_key );
+      }
+      case( DIM_ARRAY ), case( DIM_QUEUE ) {
+         $request->pack_int( $index_or_key );
+      }
+      default {
+         croak "Cannot get_property_element of a non hash, array or queue";
+      }
    }
-   elsif( $pdef->dimension == DIM_ARRAY or $pdef->dimension == DIM_QUEUE ) {
-      $request->pack_int( $index_or_key );
+
+   my $message = await $_client->request(
+      request => $request,
+   );
+
+   my $code = $message->code;
+
+   if( $code == MSG_RESULT ) {
+      return $pdef->type->unpack_value( $message );
    }
    else {
-      croak "Cannot get_property_element of a non hash, array or queue";
+      Future::Exception->throw( "Unexpected response code $code", tangence => );
    }
-
-   $client->request(
-      request => $request,
-   )->then( sub {
-      my ( $message ) = @_;
-      my $code = $message->code;
-
-      if( $code == MSG_RESULT ) {
-         my $value = $pdef->type->unpack_value( $message );
-         Future->done( $value );
-      }
-      else {
-         Future->fail( "Unexpected response code $code", tangence => );
-      }
-   });
 }
 
 =head2 prop
@@ -465,13 +405,10 @@ is not a smashed property, an exception is thrown.
 
 =cut
 
-sub prop
+method prop ( $property )
 {
-   my $self = shift;
-   my ( $property ) = @_;
-
-   if( exists $self->{props}->{$property}->{cache} ) {
-      return $self->{props}->{$property}->{cache};
+   if( exists $_props{$property}->{cache} ) {
+      return $_props{$property}->{cache};
    }
 
    croak "$self does not have a cached property '$property'";
@@ -479,17 +416,14 @@ sub prop
 
 =head2 set_property
 
-   $proxy->set_property( $prop, $value )->get
+   await $proxy->set_property( $prop, $value )
 
 Sets the value of the property in the server object.
 
 =cut
 
-sub set_property
+async method set_property ( $property, $value )
 {
-   my $self = shift;
-   my ( $property, $value ) = @_;
-
    # Detect void-context legacy uses
    defined wantarray or
       croak "->set_property in void context no longer useful - it now returns a Future";
@@ -497,34 +431,32 @@ sub set_property
    my $pdef = $self->can_property( $property )
       or croak "Class ".$self->classname." does not have a property $property";
 
-   my $client = $self->{client};
-   my $request = Tangence::Message->new( $client, MSG_SETPROP )
+   my $request = Tangence::Message->new( $_client, MSG_SETPROP )
          ->pack_int( $self->id )
          ->pack_str( $property );
    $pdef->overall_type->pack_value( $request, $value );
 
-   $client->request(
+   my $message = await $_client->request(
       request => $request,
-   )->then( sub {
-      my ( $message ) = @_;
-      my $code = $message->code;
+   );
 
-      if( $code == MSG_OK ) {
-         Future->done;
-      }
-      else {
-         Future->fail( "Unexpected response code $code", tangence => );
-      }
-   });
+   my $code = $message->code;
+
+   if( $code == MSG_OK ) {
+      return;
+   }
+   else {
+      Future::Exception->throw( "Unexpected response code $code", tangence => );
+   }
 }
 
 =head2 watch_property
 
-   $proxy->watch_property( $property, %callbacks )->get
+   await $proxy->watch_property( $property, %callbacks )
 
 =head2 watch_property_with_initial
 
-   $proxy->watch_property_with_initial( $property, %callbacks )->get
+   await $proxy->watch_property_with_initial( $property, %callbacks )
 
 Watches the given property on the server object, installing callback functions
 which will be invoked whenever the property value changes. The latter form
@@ -553,10 +485,8 @@ L<Tangence::Object>.
 
 =cut
 
-sub _watchcbs_from_args
+sub _watchcbs_from_args ( $pdef, %args )
 {
-   my ( $pdef, %args ) = @_;
-
    my $callbacks = {};
    my $on_updated = delete $args{on_updated};
    if( $on_updated ) {
@@ -575,14 +505,11 @@ sub _watchcbs_from_args
    return $callbacks;
 }
 
-sub watch_property              { shift->_watch_property( shift, 0, @_ ) }
-sub watch_property_with_initial { shift->_watch_property( shift, 1, @_ ) }
+method watch_property              { $self->_watch_property( shift, 0, @_ ) }
+method watch_property_with_initial { $self->_watch_property( shift, 1, @_ ) }
 
-sub _watch_property
+async method _watch_property ( $property, $want_initial, %args )
 {
-   my $self = shift;
-   my ( $property, $want_initial, %args ) = @_;
-
    # Detect void-context legacy uses
    defined wantarray or
       croak "->watch_property in void context no longer useful - it now returns a Future";
@@ -595,69 +522,63 @@ sub _watch_property
    # Smashed properties behave differently
    my $smash = $pdef->smashed;
 
-   if( my $cbs = $self->{props}->{$property}->{cbs} ) {
+   if( my $cbs = $_props{$property}->{cbs} ) {
       if( $want_initial and !$smash ) {
-         return $self->get_property( $property )
-            ->then( sub {
-               $callbacks->{on_set} and $callbacks->{on_set}->( $_[0] );
-               $callbacks->{on_updated} and $callbacks->{on_updated}->( $_[0] );
-               push @$cbs, $callbacks;
-               Future->done;
-            });
+         my $value = await $self->get_property( $property );
+
+         $callbacks->{on_set} and $callbacks->{on_set}->( $value );
+         $callbacks->{on_updated} and $callbacks->{on_updated}->( $value );
+         push @$cbs, $callbacks;
+         return;
       }
       elsif( $want_initial and $smash ) {
-         my $cache = $self->{props}->{$property}->{cache};
+         my $cache = $_props{$property}->{cache};
          $callbacks->{on_set} and $callbacks->{on_set}->( $cache );
          $callbacks->{on_updated} and $callbacks->{on_updated}->( $cache );
          push @$cbs, $callbacks;
-         return Future->done;
+         return;
       }
       else {
          push @$cbs, $callbacks;
-         return Future->done;
+         return;
       }
 
       die "UNREACHED";
    }
 
-   $self->{props}->{$property}->{cbs} = [ $callbacks ];
+   $_props{$property}->{cbs} = [ $callbacks ];
 
    if( $smash ) {
       if( $want_initial ) {
-         my $cache = $self->{props}->{$property}->{cache};
+         my $cache = $_props{$property}->{cache};
          $callbacks->{on_set} and $callbacks->{on_set}->( $cache );
          $callbacks->{on_updated} and $callbacks->{on_updated}->( $cache );
       }
 
-      return Future->done;
+      return;
    }
 
-   my $client = $self->{client};
-
-   my $request = Tangence::Message->new( $client, MSG_WATCH )
+   my $request = Tangence::Message->new( $_client, MSG_WATCH )
          ->pack_int( $self->id )
          ->pack_str( $property )
          ->pack_bool( $want_initial );
 
-   $client->request(
-      request => $request,
-   )->then( sub {
-      my ( $message ) = @_;
-      my $code = $message->code;
+   my $message = await $_client->request( request => $request );
 
-      if( $code == MSG_WATCHING ) {
-         Future->done;
-      }
-      else {
-         Future->fail( "Unexpected response code $code", tangence => );
-      }
-   });
+   my $code = $message->code;
+
+   if( $code == MSG_WATCHING ) {
+      return;
+   }
+   else {
+      Future::Exception->throw( "Unexpected response code $code", tangence => );
+   }
 }
 
 =head2 watch_property_with_cursor
 
    ( $cursor, $first_idx, $last_idx ) =
-      $proxy->watch_property_with_cursor( $property, $from, %callbacks )->get
+      await $proxy->watch_property_with_cursor( $property, $from, %callbacks )
 
 A variant of C<watch_property> that installs a watch on the given property of
 the server object, and additionally returns an cursor object that can be used
@@ -670,10 +591,8 @@ watch.
 
 =cut
 
-sub watch_property_with_iter
+method watch_property_with_iter
 {
-   my $self = shift;
-
    # Detect void-context legacy uses
    defined wantarray or
       croak "->watch_property_with_iter in void context no longer useful - it now returns a Future";
@@ -681,19 +600,12 @@ sub watch_property_with_iter
    return $self->watch_property_with_cursor( @_ );
 }
 
-sub watch_property_with_cursor
+async method watch_property_with_cursor ( $property, $from, %args )
 {
-   my $self = shift;
-   my ( $property, $from, %args ) = @_;
-
-   if( $from eq "first" ) {
-      $from = CUSR_FIRST;
-   }
-   elsif( $from eq "last" ) {
-      $from = CUSR_LAST;
-   }
-   else {
-      croak "Unrecognised 'from' value $from";
+   match( $from : eq ) {
+      case( "first" ) { $from = CUSR_FIRST }
+      case( "last"  ) { $from = CUSR_LAST  }
+      default         { croak "Unrecognised 'from' value $from" }
    }
 
    my $pdef = $self->can_property( $property )
@@ -704,47 +616,42 @@ sub watch_property_with_cursor
    # Smashed properties behave differently
    my $smashed = $pdef->smashed;
 
-   if( my $cbs = $self->{props}->{$property}->{cbs} ) {
+   if( my $cbs = $_props{$property}->{cbs} ) {
       die "TODO: need to synthesize a second cursor for $self";
    }
 
-   $self->{props}->{$property}->{cbs} = [ $callbacks ];
+   $_props{$property}->{cbs} = [ $callbacks ];
 
    if( $smashed ) {
       die "TODO: need to synthesize an cursor";
    }
 
-   my $client = $self->{client};
    $pdef->dimension == DIM_QUEUE or croak "Can only iterate on queue-dimension properties";
 
-   $client->request(
-      request => Tangence::Message->new( $client, MSG_WATCH_CUSR )
+   my $message = await $_client->request(
+      request => Tangence::Message->new( $_client, MSG_WATCH_CUSR )
          ->pack_int( $self->id )
          ->pack_str( $property )
          ->pack_int( $from ),
-   )->then( sub {
-      my ( $message ) = @_;
-      my $code = $message->code;
+   );
 
-      if( $code == MSG_WATCHING_CUSR ) {
-         my $cursor_id = $message->unpack_int();
-         my $first_idx = $message->unpack_int();
-         my $last_idx  = $message->unpack_int();
+   my $code = $message->code;
 
-         my $cursor = Tangence::ObjectProxy::_Cursor->new( $self, $cursor_id, $pdef->type );
-         Future->done( $cursor, $first_idx, $last_idx );
-      }
-      else {
-         Future->fail( "Unexpected response code $code", tangence => );
-      }
-   });
+   if( $code == MSG_WATCHING_CUSR ) {
+      my $cursor_id = $message->unpack_int();
+      my $first_idx = $message->unpack_int();
+      my $last_idx  = $message->unpack_int();
+
+      my $cursor = Tangence::ObjectProxy::_Cursor->new( $self, $cursor_id, $pdef->type );
+      return ( $cursor, $first_idx, $last_idx );
+   }
+   else {
+      Future::Exception->throw( "Unexpected response code $code", tangence => );
+   }
 }
 
-sub handle_request_UPDATE
+method handle_request_UPDATE ( $message )
 {
-   my $self = shift;
-   my ( $message ) = @_;
-
    my $prop  = $message->unpack_str();
    my $how   = TYPE_U8->unpack_value( $message );
 
@@ -752,7 +659,7 @@ sub handle_request_UPDATE
    my $type = $pdef->type;
    my $dim  = $pdef->dimension;
 
-   my $p = $self->{props}->{$prop} ||= {};
+   my $p = $_props{$prop} ||= {};
 
    my $dimname = DIMNAMES->[$dim];
    if( my $code = $self->can( "_update_property_$dimname" ) ) {
@@ -765,144 +672,139 @@ sub handle_request_UPDATE
    $_->{on_updated} and $_->{on_updated}->( $p->{cache} ) for @{ $p->{cbs} };
 }
 
-sub _update_property_scalar
+method _update_property_scalar ( $p, $type, $how, $message )
 {
-   my $self = shift;
-   my ( $p, $type, $how, $message ) = @_;
-
-   if( $how == CHANGE_SET ) {
-      my $value = $type->unpack_value( $message );
-      $p->{cache} = $value;
-      $_->{on_set} and $_->{on_set}->( $p->{cache} ) for @{ $p->{cbs} };
-   }
-   else {
-      croak "Change type $how is not valid for a scalar property";
-   }
-}
-
-sub _update_property_hash
-{
-   my $self = shift;
-   my ( $p, $type, $how, $message ) = @_;
-
-   if( $how == CHANGE_SET ) {
-      my $value = Tangence::Type->new( dict => $type )->unpack_value( $message );
-      $p->{cache} = $value;
-      $_->{on_set} and $_->{on_set}->( $p->{cache} ) for @{ $p->{cbs} };
-   }
-   elsif( $how == CHANGE_ADD ) {
-      my $key   = $message->unpack_str();
-      my $value = $type->unpack_value( $message );
-      $p->{cache}->{$key} = $value;
-      $_->{on_add} and $_->{on_add}->( $key, $value ) for @{ $p->{cbs} };
-   }
-   elsif( $how == CHANGE_DEL ) {
-      my $key = $message->unpack_str();
-      delete $p->{cache}->{$key};
-      $_->{on_del} and $_->{on_del}->( $key ) for @{ $p->{cbs} };
-   }
-   else {
-      croak "Change type $how is not valid for a hash property";
-   }
-}
-
-sub _update_property_queue
-{
-   my $self = shift;
-   my ( $p, $type, $how, $message ) = @_;
-
-   if( $how == CHANGE_SET ) {
-      my $value = Tangence::Type->new( list => $type )->unpack_value( $message );
-      $p->{cache} = $value;
-      $_->{on_set} and $_->{on_set}->( $p->{cache} ) for @{ $p->{cbs} };
-   }
-   elsif( $how == CHANGE_PUSH ) {
-      my @value = $message->unpack_all_sametype( $type );
-      push @{ $p->{cache} }, @value;
-      $_->{on_push} and $_->{on_push}->( @value ) for @{ $p->{cbs} };
-   }
-   elsif( $how == CHANGE_SHIFT ) {
-      my $count = $message->unpack_int();
-      splice @{ $p->{cache} }, 0, $count, ();
-      $_->{on_shift} and $_->{on_shift}->( $count ) for @{ $p->{cbs} };
-   }
-   else {
-      croak "Change type $how is not valid for a queue property";
-   }
-}
-
-sub _update_property_array
-{
-   my $self = shift;
-   my ( $p, $type, $how, $message ) = @_;
-
-   if( $how == CHANGE_SET ) {
-      my $value = Tangence::Type->new( list => $type )->unpack_value( $message );
-      $p->{cache} = $value;
-      $_->{on_set} and $_->{on_set}->( $p->{cache} ) for @{ $p->{cbs} };
-   }
-   elsif( $how == CHANGE_PUSH ) {
-      my @value = $message->unpack_all_sametype( $type );
-      push @{ $p->{cache} }, @value;
-      $_->{on_push} and $_->{on_push}->( @value ) for @{ $p->{cbs} };
-   }
-   elsif( $how == CHANGE_SHIFT ) {
-      my $count = $message->unpack_int();
-      splice @{ $p->{cache} }, 0, $count, ();
-      $_->{on_shift} and $_->{on_shift}->( $count ) for @{ $p->{cbs} };
-   }
-   elsif( $how == CHANGE_SPLICE ) {
-      my $start = $message->unpack_int();
-      my $count = $message->unpack_int();
-      my @value = $message->unpack_all_sametype( $type );
-      splice @{ $p->{cache} }, $start, $count, @value;
-      $_->{on_splice} and $_->{on_splice}->( $start, $count, @value ) for @{ $p->{cbs} };
-   }
-   elsif( $how == CHANGE_MOVE ) {
-      my $index = $message->unpack_int();
-      my $delta = $message->unpack_int();
-      # it turns out that exchanging neighbours is quicker by list assignment,
-      # but other times it's generally best to use splice() to extract then
-      # insert
-      if( abs($delta) == 1 ) {
-         @{$p->{cache}}[$index,$index+$delta] = @{$p->{cache}}[$index+$delta,$index];
+   match( $how : == ) {
+      case( CHANGE_SET ) {
+         my $value = $type->unpack_value( $message );
+         $p->{cache} = $value;
+         $_->{on_set} and $_->{on_set}->( $p->{cache} ) for @{ $p->{cbs} };
       }
-      else {
-         my $elem = splice @{ $p->{cache} }, $index, 1, ();
-         splice @{ $p->{cache} }, $index + $delta, 0, ( $elem );
+      default {
+         croak "Change type $how is not valid for a scalar property";
       }
-      $_->{on_move} and $_->{on_move}->( $index, $delta ) for @{ $p->{cbs} };
-   }
-   else {
-      croak "Change type $how is not valid for an array property";
    }
 }
 
-sub _update_property_objset
+method _update_property_hash ( $p, $type, $how, $message )
 {
-   my $self = shift;
-   my ( $p, $type, $how, $message ) = @_;
+   match( $how : == ) {
+      case( CHANGE_SET ) {
+         my $value = Tangence::Type->make( dict => $type )->unpack_value( $message );
+         $p->{cache} = $value;
+         $_->{on_set} and $_->{on_set}->( $p->{cache} ) for @{ $p->{cbs} };
+      }
+      case( CHANGE_ADD ) {
+         my $key   = $message->unpack_str();
+         my $value = $type->unpack_value( $message );
+         $p->{cache}->{$key} = $value;
+         $_->{on_add} and $_->{on_add}->( $key, $value ) for @{ $p->{cbs} };
+      }
+      case( CHANGE_DEL ) {
+         my $key = $message->unpack_str();
+         delete $p->{cache}->{$key};
+         $_->{on_del} and $_->{on_del}->( $key ) for @{ $p->{cbs} };
+      }
+      default {
+         croak "Change type $how is not valid for a hash property";
+      }
+   }
+}
 
-   if( $how == CHANGE_SET ) {
-      # Comes across in a LIST. We need to map id => obj
-      my $objects = Tangence::Type->new( list => $type )->unpack_value( $message );
-      $p->{cache} = { map { $_->id => $_ } @$objects };
-      $_->{on_set} and $_->{on_set}->( $p->{cache} ) for @{ $p->{cbs} };
+method _update_property_queue ( $p, $type, $how, $message )
+{
+   match( $how : == ) {
+      case( CHANGE_SET ) {
+         my $value = Tangence::Type->make( list => $type )->unpack_value( $message );
+         $p->{cache} = $value;
+         $_->{on_set} and $_->{on_set}->( $p->{cache} ) for @{ $p->{cbs} };
+      }
+      case( CHANGE_PUSH ) {
+         my @value = $message->unpack_all_sametype( $type );
+         push @{ $p->{cache} }, @value;
+         $_->{on_push} and $_->{on_push}->( @value ) for @{ $p->{cbs} };
+      }
+      case( CHANGE_SHIFT ) {
+         my $count = $message->unpack_int();
+         splice @{ $p->{cache} }, 0, $count, ();
+         $_->{on_shift} and $_->{on_shift}->( $count ) for @{ $p->{cbs} };
+      }
+      default {
+         croak "Change type $how is not valid for a queue property";
+      }
    }
-   elsif( $how == CHANGE_ADD ) {
-      # Comes as object only
-      my $obj = $type->unpack_value( $message );
-      $p->{cache}->{$obj->id} = $obj;
-      $_->{on_add} and $_->{on_add}->( $obj ) for @{ $p->{cbs} };
+}
+
+method _update_property_array ( $p, $type, $how, $message )
+{
+   match( $how : == ) {
+      case( CHANGE_SET ) {
+         my $value = Tangence::Type->make( list => $type )->unpack_value( $message );
+         $p->{cache} = $value;
+         $_->{on_set} and $_->{on_set}->( $p->{cache} ) for @{ $p->{cbs} };
+      }
+      case( CHANGE_PUSH ) {
+         my @value = $message->unpack_all_sametype( $type );
+         push @{ $p->{cache} }, @value;
+         $_->{on_push} and $_->{on_push}->( @value ) for @{ $p->{cbs} };
+      }
+      case( CHANGE_SHIFT ) {
+         my $count = $message->unpack_int();
+         splice @{ $p->{cache} }, 0, $count, ();
+         $_->{on_shift} and $_->{on_shift}->( $count ) for @{ $p->{cbs} };
+      }
+      case( CHANGE_SPLICE ) {
+         my $start = $message->unpack_int();
+         my $count = $message->unpack_int();
+         my @value = $message->unpack_all_sametype( $type );
+         splice @{ $p->{cache} }, $start, $count, @value;
+         $_->{on_splice} and $_->{on_splice}->( $start, $count, @value ) for @{ $p->{cbs} };
+      }
+      case( CHANGE_MOVE ) {
+         my $index = $message->unpack_int();
+         my $delta = $message->unpack_int();
+         # it turns out that exchanging neighbours is quicker by list assignment,
+         # but other times it's generally best to use splice() to extract then
+         # insert
+         if( abs($delta) == 1 ) {
+            @{$p->{cache}}[$index,$index+$delta] = @{$p->{cache}}[$index+$delta,$index];
+         }
+         else {
+            my $elem = splice @{ $p->{cache} }, $index, 1, ();
+            splice @{ $p->{cache} }, $index + $delta, 0, ( $elem );
+         }
+         $_->{on_move} and $_->{on_move}->( $index, $delta ) for @{ $p->{cbs} };
+      }
+      default {
+         croak "Change type $how is not valid for an array property";
+      }
    }
-   elsif( $how == CHANGE_DEL ) {
-      # Comes as ID number only
-      my $id = $message->unpack_int();
-      delete $p->{cache}->{$id};
-      $_->{on_del} and $_->{on_del}->( $id ) for @{ $p->{cbs} };
-   }
-   else {
-      croak "Change type $how is not valid for an objset property";
+}
+
+method _update_property_objset ( $p, $type, $how, $message )
+{
+   match( $how : == ) {
+      case( CHANGE_SET ) {
+         # Comes across in a LIST. We need to map id => obj
+         my $objects = Tangence::Type->make( list => $type )->unpack_value( $message );
+         $p->{cache} = { map { $_->id => $_ } @$objects };
+         $_->{on_set} and $_->{on_set}->( $p->{cache} ) for @{ $p->{cbs} };
+      }
+      case( CHANGE_ADD ) {
+         # Comes as object only
+         my $obj = $type->unpack_value( $message );
+         $p->{cache}->{$obj->id} = $obj;
+         $_->{on_add} and $_->{on_add}->( $obj ) for @{ $p->{cbs} };
+      }
+      case( CHANGE_DEL ) {
+         # Comes as ID number only
+         my $id = $message->unpack_int();
+         delete $p->{cache}->{$id};
+         $_->{on_del} and $_->{on_del}->( $id ) for @{ $p->{cbs} };
+      }
+      default {
+         croak "Change type $how is not valid for an objset property";
+      }
    }
 }
 
@@ -915,20 +817,16 @@ previously installed using C<watch_property>.
 
 =cut
 
-sub unwatch_property
+method unwatch_property ( $property )
 {
-   my $self = shift;
-   my ( $property ) = @_;
-
    $self->can_property( $property )
       or croak "Class ".$self->classname." does not have a property $property";
 
    # TODO: mark cursors as destroyed and invalid
-   delete $self->{props}->{$property};
+   delete $_props{$property};
 
-   my $client = $self->{client};
-   $client->request(
-      request => Tangence::Message->new( $client, MSG_UNWATCH )
+   $_client->request(
+      request => Tangence::Message->new( $_client, MSG_UNWATCH )
          ->pack_int( $self->id )
          ->pack_str( $property ),
 
@@ -936,10 +834,10 @@ sub unwatch_property
    );
 }
 
-package # hide from index
-   Tangence::ObjectProxy::_Cursor;
-use Carp;
-use Tangence::Constants;
+class Tangence::ObjectProxy::_Cursor
+{
+   use Carp;
+   use Tangence::Constants;
 
 =head1 CURSOR METHODS
 
@@ -948,37 +846,37 @@ by the C<watch_property_with_cursor> method.
 
 =cut
 
-sub new
-{
-   my $class = shift;
-   return bless [ @_ ], $class;
-}
+   has $obj          :param :reader;
+   has $id           :param :reader;
+   has $element_type :param;
 
-sub obj    { shift->[0] }
-sub id     { shift->[1] }
-sub client { shift->obj->{client} }
+   sub BUILDARGS ( $class, $obj, $id, $element_type )
+   {
+      return ( obj => $obj, id => $id, element_type => $element_type );
+   }
 
-sub DESTROY
-{
-   my $self = shift;
+   method client { $obj->client }
 
-   return unless $self->obj and my $id = $self->id and my $client = $self->client;
+   # TODO: Object::Pad probably should do this bit
+   method DESTROY
+   {
+      return unless $obj and my $client = $self->client;
 
-   $client->request(
-      request => Tangence::Message->new( $client, MSG_CUSR_DESTROY )
-         ->pack_int( $id ),
+      $client->request(
+         request => Tangence::Message->new( $client, MSG_CUSR_DESTROY )
+            ->pack_int( $id ),
 
-      on_response => sub {},
-   );
-}
+         on_response => sub {},
+      );
+   }
 
 =head2 next_forward
 
-   ( $index, @more ) = $cursor->next_forward( $count )->get
+   ( $index, @more ) = await $cursor->next_forward( $count )
 
 =head2 next_backward
 
-   ( $index, @more ) = $cursor->next_backward( $count )->get
+   ( $index, @more ) = await $cursor->next_backward( $count )
 
 Requests the next items from the cursor. C<next_forward> moves forwards
 towards higher-numbered indices, and C<next_backward> moves backwards towards
@@ -992,52 +890,43 @@ new elements if the cursor is already at the end.
 
 =cut
 
-sub next_forward
-{
-   my $self = shift;
-   $self->_next( CUSR_FWD, @_ );
-}
+   method next_forward
+   {
+      $self->_next( CUSR_FWD, @_ );
+   }
 
-sub next_backward
-{
-   my $self = shift;
-   $self->_next( CUSR_BACK, @_ );
-}
+   method next_backward
+   {
+      $self->_next( CUSR_BACK, @_ );
+   }
 
-sub _next
-{
-   my $self = shift;
-   my ( $direction, $count ) = @_;
+   async method _next ( $direction, $count = 1 )
+   {
+      # Detect void-context legacy uses
+      defined wantarray or
+         croak "->next_forward/backward in void context no longer useful - it now returns a Future";
 
-   # Detect void-context legacy uses
-   defined wantarray or
-      croak "->next_forward/backward in void context no longer useful - it now returns a Future";
+      my $client = $self->client;
 
-   my $obj = $self->obj;
-   my $id  = $self->id;
-   my $element_type = $self->[2];
+      my $message = await $client->request(
+         request => Tangence::Message->new( $client, MSG_CUSR_NEXT )
+            ->pack_int( $id )
+            ->pack_int( $direction )
+            ->pack_int( $count || 1 ),
+      );
 
-   my $client = $self->client;
-
-   $client->request(
-      request => Tangence::Message->new( $client, MSG_CUSR_NEXT )
-         ->pack_int( $id )
-         ->pack_int( $direction )
-         ->pack_int( $count || 1 ),
-   )->then( sub {
-      my ( $message ) = @_;
       my $code = $message->code;
 
       if( $code == MSG_CUSR_RESULT ) {
-         Future->done(
+         return (
             $message->unpack_int(),
             $message->unpack_all_sametype( $element_type ),
          );
       }
       else {
-         Future->fail( "Unexpected response code $code", tangence => );
+         Future::Exception->throw( "Unexpected response code $code", tangence => );
       }
-   });
+   }
 }
 
 =head1 AUTHOR
